/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : PowerMonitor_Platformservice.c
   Author             : Arjun
   Version            : V0.0.0.1
   Origination Date   : 25/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Power Monitor platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             25/02/2022			Arjun

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */


/* USER CODE END Macros */


/* USER CODE BEGIN Global Variables */


#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)
extern pvt_variable pv;


#if (SDK_SERVICE_PM == PS_ENABLE)

/** @brief IGN deinit
 *
 *  This function deinit the IGN pin.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void IGN_deinit(void)
{
	/* Deinitialize IGN pin */
	HAL_GPIO_DeInit(GPIOC, GPIO_PIN_4);

	/* Disable the IGN interrupt */
	HAL_NVIC_DisableIRQ(EXTI4_IRQn);
}

/** @brief IGN Wakeup Source
 *
 *  This function configures the IGN
 *  as the wakeup source for the Power Management.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void IGN_Wakeup_Source(void)
{
	/* validate the IGN wakeup state */
#if(IGN_WAKEUP_SOURCE == PS_ENABLE)

	GPIO_InitTypeDef GPIO_InitStruct = {0};

	GPIO_InitStruct.Pin = GPIO_PIN_4;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI4_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI4_IRQn);

#endif
}

/** @brief IGN reinit
 *
 *  This function reinit the IGN pin.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void IGN_reinit(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	GPIO_InitStruct.Pin = GPIO_PIN_4;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI4_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI4_IRQn);
}
#endif


/** @brief Power Monitor DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return  0 : PS_SUCCESS
 *          -1 : PS_FAILURE
 *
 */
int8_t PowerMonitor_DB_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	/*Configure GPIO pin : PC4 */
	GPIO_InitStruct.Pin = GPIO_PIN_4;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

	return PS_SUCCESS;
}

/** @brief PowerMonitor Timer Callback
 *
 *  This function is used for the PowerMonitor Sampling time
 *  functionality
 *
 *  @param  xTimer : PowerMonitor timer handle
 *
 *  @return None
 *
 */
void PMTimerCallback( TimerHandle_t xTimer )
{
	/* Notify to the PM task with PM timer callback */
	xTaskNotify( pv.pmonitor_task, (1<<POWERMONITOR_TMR_EVENT), eSetBits );
}

/** @brief Read PowerMonitor Status
 *
 *  This function reads the Power Monitor Status.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void read_powermonitor_status(void)
{
	uint16_t read_val = 0;
	uint16_t battery;
	float temperature = 0;
	uint16_t Vref_int = 0;

	/* raed and update the state value */
	read_val = HAL_GPIO_ReadPin(IGN_DATA_PORT, IGN_DATA_PIN);
	Set_DL(IGNITION_STATUS, (uint8_t *)&read_val);

	/* read and update the Battery value */
	read_val = GetADC_Results(BATTERY_CHANNEL);
	/* Battery level in Millivoltage */

	battery = (uint16_t)(read_val * 10.12);

	Set_DL(BATTERY_LEVEL, (uint8_t *)&battery);

	/*read and update the vref value*/
	read_val = GetADC_Results(VREF_CHANNEL);

	Vref_int = __HAL_ADC_CALC_VREFANALOG_VOLTAGE(read_val, ADC_RESOLUTION_12B);

	/* read and update the Thermostat value */
	read_val = GetADC_Results(TEMPERATURE_CHANNEL);

	/* Temperature in degree celsius */
	temperature = __HAL_ADC_CALC_TEMPERATURE(Vref_int, read_val, ADC_RESOLUTION_12B);

	if(temperature < TEMP_SENSOR_MIN_VAL)
	{
		temperature = TEMP_SENSOR_MIN_VAL; /*Clamp to min value*/
	}
	Set_DL(THERMOSTAT_LEVEL, (uint8_t *)&temperature);

}
/** @brief Power Monitor Platform service task
 *
 *  This function starts the Power Monitor Platform Service
 *  task.
 *
 *  @param  pvParameters : Task Private structure
 *
 *  @return none
 *
 */
void PowerMonitor_ServiceStart(void *pvParameters)
{
	uint32_t cNotifiedValue;
	uint8_t *pmonitorbuf;
	uint8_t res;
	uint8_t func;

	/* Start POWERMONITOR timer */
	xTimerStart( pv.xTimers[ PS_POWERMONITOR_TIMER ], 0 );

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_POWERMONITOR_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("Power Monitor Task failed to ping the WD Timer\r\n");
		}
#endif
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_POWER_MONITOR_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		if (cNotifiedValue & (1<<POWERMONITOR_TMR_EVENT))
		{
			/* Read and update the power monitor status */
			read_powermonitor_status();
		}

		if (cNotifiedValue & (1<<POWERMONITOR_MSG_EVENT))
		{
				/* Receive Message from the Power Monitor SDK API */
				res = xQueueReceive(pv.pmonitortxq, &pmonitorbuf , ( TickType_t ) Q_TICKSTOWAIT );

				if(res != pdTRUE)
				{
					break;
				}

				func = pmonitorbuf[PWR_MONITOR_FUNC_INDEX];

				/* Update the result */
				pmonitorbuf[PWR_MONITOR_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case GET_BATTERY_LEVEL:
						pmonitorbuf[PWR_MONITOR_GET_VAL] = GetADC_Results(BATTERY_CHANNEL);
						break;

					case GET_IGN_STATUS:
						/* Read the IGN Pin state */
						pmonitorbuf[PWR_MONITOR_GET_VAL] = HAL_GPIO_ReadPin(IGN_DATA_PORT, IGN_DATA_PIN);
						break;

					case GET_THERMOSTAT_LEVEL:
						pmonitorbuf[PWR_MONITOR_GET_VAL] = GetADC_Results(TEMPERATURE_CHANNEL);
						break;

					default:
						pmonitorbuf[PWR_MONITOR_RESULT_INDEX] = FAILURE;
						break;
				}

				/* Send the response to the SDK API */
				xQueueSend( pv.pmonitorrxq, ( void * )&pmonitorbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start Power Monitor Platform service
 *
 *  This function initializes the following modules
 *     ===> Power Monitor DB
 *     ===> Power Monitor platform driver
 *     ===> Power Monitor software resources
 *     ===> Start the Power Monitor service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t PowerMonitor_PS_Start(void)
{
	BaseType_t xReturned;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(IGN_deinit,IGN_Wakeup_Source,IGN_reinit);
#endif

	/* Create the Power Monitor timer */
	pv.xTimers[ PS_POWERMONITOR_TIMER ] = xTimerCreate
	                   ( "PowerMonitor_Timer",
	                     PS_POWERMONITOR_TIMEOUT,
	                     pdTRUE,
	                     ( void * ) 0,
	                     PMTimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_POWERMONITOR_TIMER ])
	{
		LOGERROR("Unable to initialize the PowerMonitor timer\r\n");
		return PS_FAILURE;
	}

	/* Create Power Monitor TxQ for receiving value from the SDK API */
	pv.pmonitortxq = xQueueCreate( POWERMONITOR_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.pmonitortxq)
	{
		LOGERROR("Unable to initialize the Power Monitor TxQ\r\n");
		return PS_FAILURE;
	}

	/* Create Power Monitor RxQ for sending return value to the SDK API */
	pv.pmonitorrxq = xQueueCreate( POWERMONITOR_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.pmonitorrxq)
	{
		LOGERROR("Unable to initialize the Power Monitor RxQ\r\n");
		return PS_FAILURE;
	}

	/* Start the Power Monitor Platform service thread */
	xReturned = xTaskCreate(PowerMonitor_ServiceStart, "Power_Monitor",
			512, NULL, PS_POWER_MONITOR_TASK_PRIORITY,   &pv.pmonitor_task);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the Power Monitor Thread\r\n");
		return PS_FAILURE;
	}

	LOGERROR("Power Monitor Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif	//SDK_SERVICE_POWER_MONITOR

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
