/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : RTC_platformservice.c
   Author             : Arjun
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the RTC platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Arjun

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"

/* USER CODE END Includes */

/* USER CODE BEGIN Macros */


/* USER CODE END Macros */


/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_RTC == PS_ENABLE)
extern pvt_variable pv;
/* USER CODE BEGIN Global Variables */

#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief RTC Wakeup Source
 *
 *  This function configures the RTC
 *  as the wakeup source for the Power Management.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void RTC_Wakeup_Source(void)
{
	/* validate the RTC wakeup state */
#if(RTC_WAKEUP_SOURCE_STATE == PS_ENABLE)

	uint32_t timeout = 0;

	/* Disable all used wakeup source */
	HAL_RTCEx_DeactivateWakeUpTimer(pv.hrtc);

	/* get the wakeup timer from the DB */
	Get_DL(RTC_WAKEUP_TIMEOUT, (uint8_t *)&timeout);

	/* To configure the wake up timer */
	HAL_RTCEx_SetWakeUpTimer_IT(pv.hrtc, timeout, RTC_WAKEUPCLOCK_RTCCLK_DIV16);
#endif

}
#endif

/**
  * @brief  Alarm_A callback
  *
  *     This function will be called when the Alarm_A event occured.
  *
  * @param  hrtc : RTC handle
  *
  * @retval None
  */
void HAL_RTC_AlarmAEventCallback(RTC_HandleTypeDef *hrtc)
{
	/* Send Notify to the RTC task to update the Alarm_A Status */
	xTaskNotifyFromISR( pv.rtc_task, (1<<RTC_ALARM_A_OCCURED) , eSetBits, NULL );
}

/**
  * @brief  Alarm_B callback
  *
  *     This function will be called when the Alarm_B event occured.
  *
  * @param  hrtc : RTC handle
  *
  * @retval None
  */
void HAL_RTCEx_AlarmBEventCallback(RTC_HandleTypeDef * hrtc)
{
	/* Send Notify to the RTC task to update the Alarm_B Status */
	xTaskNotifyFromISR( pv.rtc_task, (1<<RTC_ALARM_B_OCCURED) , eSetBits, NULL );
}

/** @brief RTC DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void RTC_PS_DB_Init(void)
{
	uint8_t val;

	/* Read the RTC from the platform HAL Layer */
	HAL_RTC_GetDate(pv.hrtc, &pv.sDate, RTC_FORMAT_BIN);
	HAL_RTC_GetTime(pv.hrtc, &pv.sTime, RTC_FORMAT_BIN);

	/* Update the DB variables with RTC Time */
    Set_DL(RTC_SECOND, &pv.sTime.Seconds);
    Set_DL(RTC_MINUTE, &pv.sTime.Minutes);
    Set_DL(RTC_HOUR, &pv.sTime.Hours);
    Set_DL(RTC_DATE, &pv.sDate.Date);
    Set_DL(RTC_WEEK_DAY, &pv.sDate.WeekDay);
    Set_DL(RTC_MONTH, &pv.sDate.Month);
    Set_DL(RTC_YEAR, &pv.sDate.Year);
    Set_DL(RTC_TIME_FORMAT,&pv.sTime.TimeFormat);

    /* update the RTC time */
    val = SET_RTC_PARAMETER;
    Set_DL(SET_RTC, &val);
}


/** @brief RTC Timer Callback
 *
 *  This function is used for updating the timer into the RTC DB
 *
 *  @param  xTimer : RTC timer handle
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
void RTCTimerCallback( TimerHandle_t xTimer )
{
	/* Notify to the RTC task to Update the DB */
	xTaskNotify( pv.rtc_task, (1<<RTC_TMR_EVENT), eSetBits );
}

/** @brief RTC Platform service task
 *
 *  This function starts the RTC Platform Service
 *  task.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
static void RTC_ServiceStart(void * pvParameters)
{
	uint32_t cNotifiedValue;
	uint8_t *rtcbuf;
	uint8_t res;
	uint8_t func;

	/* STart the timer for the RTC update */
	xTimerStart(pv.xTimers[ PS_RTC_TIMER ], 0);

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_RTC_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("RTC Task failed to ping the WD Timer\r\n");
		}
#endif

		/* Wait for receiving the Event from the SDK API and RTC timer */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_RTC_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Event from SDK Light Sensor API */
		if (cNotifiedValue & (1<<RTC_TMR_EVENT))
		{
			/* Read the RTC from the platform HAL Layer */
			HAL_RTC_GetTime(pv.hrtc, &pv.sTime, RTC_FORMAT_BIN);
			HAL_RTC_GetDate(pv.hrtc, &pv.sDate, RTC_FORMAT_BIN);

			/* Update the DB variables with RTC Time */
			Set_DL(GET_RTC_SECOND, &pv.sTime.Seconds);
			Set_DL(GET_RTC_MINUTE, &pv.sTime.Minutes);
			Set_DL(GET_RTC_HOUR, &pv.sTime.Hours);
			Set_DL(GET_RTC_DATE, &pv.sDate.Date);
			Set_DL(GET_RTC_WEEK_DAY, &pv.sDate.WeekDay);
			Set_DL(GET_RTC_MONTH, &pv.sDate.Month);
		    Set_DL(GET_RTC_YEAR, &pv.sDate.Year);
		    Set_DL(RTC_TIME_FORMAT,&pv.sTime.TimeFormat);

		}

		if (cNotifiedValue & (1<<RTC_ALARM_A_OCCURED))
		{
			res = ALARM_OCCURED;

			/* Set the Alarm Status into the DB */
			Set_DL(ALARM_A_STATUS, &res);
		}

		if (cNotifiedValue & (1<<RTC_ALARM_B_OCCURED))
		{
			res = ALARM_OCCURED;

			/* Set the Alarm Status into the DB */
			Set_DL(ALARM_B_STATUS, &res);
		}

		if (cNotifiedValue & (1<< RTC_MSG_EVENT))
		{
				/* Receive Message from the RTC Sensor SDK API */
				res = xQueueReceive(pv.rtctxq, &rtcbuf , ( TickType_t ) Q_TICKSTOWAIT );
				if(res != pdTRUE)
				{
					break;
				}

				func = rtcbuf[RTC_FUNC_INDEX];
				/* Update the result */
				rtcbuf[RTC_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case RTC_GET:
						/* Read the RTC from the platform HAL Layer */
						HAL_RTC_GetDate(pv.hrtc, &pv.sDate, RTC_FORMAT_BIN);
						HAL_RTC_GetTime(pv.hrtc, &pv.sTime, RTC_FORMAT_BIN);

						/* Return the data and time to the SDK API */
						rtcbuf[RTC_SEC_INDEX] = pv.sTime.Seconds;
						rtcbuf[RTC_MIN_INDEX] = pv.sTime.Minutes;
						rtcbuf[RTC_HOUR_INDEX] = pv.sTime.Hours;
						rtcbuf[RTC_DATE_INDEX] = pv.sDate.Date;
						rtcbuf[RTC_MONTH_INDEX] = pv.sDate.Month;
						rtcbuf[RTC_YEAR_INDEX] = pv.sDate.Year;
						rtcbuf[RTC_WEEKDAY_INDEX] = pv.sDate.WeekDay;
						break;

					case RTC_SET:

						/* Reinitialize RTC Init */
						HAL_RTC_DeInit(pv.hrtc);

						pv.hrtc->Instance = RTC;

						/* Get and Update the RTC Time Format */
						if(FORMAT_12_HOUR == rtcbuf[RTC_TIME_FORMAT_INDEX])
						{
							pv.hrtc->Init.HourFormat = RTC_HOURFORMAT_12;
						}
						else
						{
							pv.hrtc->Init.HourFormat = RTC_HOURFORMAT_24;
						}

						pv.hrtc->Init.AsynchPrediv = 127;
						pv.hrtc->Init.SynchPrediv = 255;
						pv.hrtc->Init.OutPut = RTC_OUTPUT_DISABLE;
						pv.hrtc->Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
						pv.hrtc->Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;
						pv.hrtc->Init.OutPutRemap = RTC_OUTPUT_REMAP_NONE;
						if(HAL_RTC_Init(pv.hrtc) != HAL_OK)
						{
							Error_Handler();
						}

						/* Update the RTC variable */
						pv.sTime.Hours = rtcbuf[RTC_HOUR_INDEX];
						pv.sTime.Minutes = rtcbuf[RTC_MIN_INDEX];
						pv.sTime.Seconds = rtcbuf[RTC_SEC_INDEX];
						pv.sTime.TimeFormat = rtcbuf[RTC_TIME_FORMAT_INDEX];
						if(HAL_RTC_SetTime(pv.hrtc, &pv.sTime, RTC_FORMAT_BIN) != HAL_OK)
						{
							Error_Handler();
						}

						/* Update the RTC variable */
						pv.sDate.Date = rtcbuf[RTC_DATE_INDEX];
						pv.sDate.Month = rtcbuf[RTC_MONTH_INDEX];
						pv.sDate.Year = rtcbuf[RTC_YEAR_INDEX];
						pv.sDate.WeekDay = rtcbuf[RTC_WEEKDAY_INDEX];
						if(HAL_RTC_SetDate(pv.hrtc, &pv.sDate, RTC_FORMAT_BIN) != HAL_OK)
						{
							Error_Handler();
						}

						break;

					case RTC_SET_ALARM_A:
					case RTC_SET_ALARM_B:

						/* Set the proper Alarm module name*/
						if (RTC_SET_ALARM_A == func)
							pv.sAlarm.Alarm = RTC_ALARM_A;
						else
							pv.sAlarm.Alarm = RTC_ALARM_B;

						/* Deactivate the Alarm */
						HAL_RTC_DeactivateAlarm(pv.hrtc, pv.sAlarm.Alarm);

						/** Enable the Alarm A
						 */
						pv.sAlarm.AlarmTime.Hours = rtcbuf[ALARM_HOUR_INDEX];
						pv.sAlarm.AlarmTime.Minutes = rtcbuf[ALARM_MIN_INDEX];
						pv.sAlarm.AlarmTime.Seconds = rtcbuf[ALARM_SEC_INDEX];
						pv.sAlarm.AlarmTime.SubSeconds = 0x0;
						pv.sAlarm.AlarmTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
						pv.sAlarm.AlarmTime.StoreOperation = RTC_STOREOPERATION_RESET;
						pv.sAlarm.AlarmMask = RTC_ALARMMASK_NONE;
						pv.sAlarm.AlarmSubSecondMask = RTC_ALARMSUBSECONDMASK_ALL;
						pv.sAlarm.AlarmDateWeekDaySel = RTC_ALARMDATEWEEKDAYSEL_WEEKDAY;
						pv.sAlarm.AlarmDateWeekDay = rtcbuf[ALARM_WEEKDAY_INDEX];

						/* Call HAL Layer Function */
						if (HAL_RTC_SetAlarm_IT(pv.hrtc, &pv.sAlarm, RTC_FORMAT_BIN) != HAL_OK)
						{
							/* Initialization Error */
							Error_Handler();
						}

					    break;

					default:

						/* Update the result */
						rtcbuf[RTC_RESULT_INDEX] = FAILURE;
						break;
				}
				/* Send the response to the SDK API */
				xQueueSend( pv.rtcrxq, ( void * )&rtcbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}


/** @brief Start RTC Platform service
 *
 *  This function initializes the following modules
 *     ===> RTC DB
 *     ===> RTC platform driver
 *     ===> RTC software resources
 *     ===> Start the RTC service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t RTC_PS_Start(void)
{
	BaseType_t xReturned;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(NULL,RTC_Wakeup_Source,NULL);
#endif

	/* Create the RTC timer for the update DB */
	pv.xTimers[ PS_RTC_TIMER ] = xTimerCreate
	                   ( "RTC_Timer",
	                	 PS_RTC_SECOND_TIMEOUT,
	                     pdTRUE,
	                     ( void * ) 0,
	                     RTCTimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_RTC_TIMER ])
	{
		LOGERROR("Unable to initialize the RTC timer\r\n");
		return PS_FAILURE;
	}


	/* Create RTC TxQ for receiving value from the SDK API */
	pv.rtctxq = xQueueCreate( RTC_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.rtctxq)
	{
		LOGERROR("Unable to create the RTC TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create RTC RxQ for sending return value to the SDK API */
	pv.rtcrxq = xQueueCreate( RTC_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.rtcrxq)
	{
		LOGERROR("Unable to create the RTC RxQ \r\n");
		return PS_FAILURE;
	}

	/* Start the RTC Platform service thread */
	xReturned = xTaskCreate(RTC_ServiceStart, "RTC",
			RTC_STACK_DEPTH, NULL, PS_RTC_TASK_PRIORITY, &pv.rtc_task);

	if( xReturned != pdPASS )
	{
		LOGERROR("Unable to create the RTC Task \r\n");
		return PS_FAILURE;
	}

	LOGERROR("RTC Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif	//SDK_SERVICE_RTC

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
